%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0. Basic settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; 
clc;
disp_op    = 'off';  % show fsolve iteration
sm      = 1d-7;      % small number for log or devide
sm1     = 1d-7;      % convergence criterion for value function iteration
sm2     = 1d-7;      % convergence criterion for SS distribution iteration
sm3     = 1d-8;      % convergence criterion for fsolve
num_y_s = 50;        % number of income grids
num_y   = 100;       % number of income grids
max_w   = 50;        % maximum value of waiting list
n_burn  = 50;
x_guess =  [ 20.3556026335767         0.257417319942105];
y_low   = 52367.227;
tau     = 0.3;       % tax rate in public housing
p_m     = 1;         % market rent
p_r   = 1 -0.51259773 ;
k_p   = 0.0990;
k_r   = 0.5720;
k_m   = 1-k_p-k_r;
rho   = 0.6661;
% generate randome sample 
n_sim         = 5000;      % number of simulation sample
n_t           = 100;       % length of simulation
rng(10) % random number seed
qt_draw = rand(n_sim,n_t);
pt_draw = rand(n_sim,n_t);
yt_draw = rand(n_sim,n_t);
% read data and parameter values
data = xlsread('moment_baseline.xlsx')';
load('test_baseline.mat')
x=xx;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Setting up
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read input parameters
h     = x(1);
h_r   = x(2);
mu    = x(3);
sigma = x(4);
alpha = x(5);
gamma = x(6);
h_min = x(7);

y_min = h_min*p_m;

% algorithm stabilization tricks
sigma = max(sm,sigma);
mu = max(sm,mu);

% Tauchen small grids
m =4;             % coverage m standard deviation
[log_y_s,g_tran_s] = tauchen(num_y_s,mu,rho,sigma,m); % (log income, transition matrix)
y_s = exp(log_y_s)+y_min;   % income level

% define flow utility function
u_p_s = ((1-tau)*y_s).^(1-alpha).*(h-h_min).^alpha;            % public
u_r_s = zeros(num_y_s,1);                                            % regulated
u_m_s = alpha^alpha*(1-alpha)^(1-alpha)*(y_s-h_min*p_m)/p_m^alpha; % mover
h_m_s = alpha*(y_s-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s_s = zeros(num_y_s,num_y_s);                                      % stayer

for i=1:num_y_s
    for j=1:num_y_s
        if y_s(i)-p_m*h_m_s(j) < sm
            u_s_s(i,j) = -1000000000;
        else
            u_s_s(i,j) = (y_s(i)-p_m*h_m_s(j))^(1-alpha)*(h_m_s(j)-h_min)^alpha;
        end
    end
end


for i=1:num_y_s
    if y_s(i)-p_r*h_r < sm
        u_r_s(i) = -1000000000;
    else
        u_r_s(i) = (y_s(i)-p_r*h_r)^(1-alpha)*(h_r-h_min)^alpha;
    end
end
 
%Tauchen approximation of AR1
m =4;             % coverage m standard deviation
[log_y,g_tran] = tauchen(num_y,mu,rho,sigma,m); % (log income, transition matrix)
y = exp(log_y)+y_min;   % income level

y_cdf = zeros(num_y,num_y); % CDF of income process
for i=1:num_y
    y_cdf(:,i) = sum(g_tran(:,1:i),2);
end

g_mat = g_tran';

% define flow utility function
u_p = ((1-tau)*y).^(1-alpha).*(h-h_min).^alpha;            % public
u_r = zeros(num_y,1);                                            % regulated
u_m = alpha^alpha*(1-alpha)^(1-alpha)*(y-h_min*p_m)/p_m^alpha; % mover
h_m = alpha*(y-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s = zeros(num_y,num_y);                                      % stayer

for i=1:num_y
    for j=1:num_y
        if y(i)-p_m*h_m(j) < sm
            u_s(i,j) = -1000000000;
        else
            u_s(i,j) = (y(i)-p_m*h_m(j))^(1-alpha)*(h_m(j)-h_min)^alpha;
        end
    end
end


for i=1:num_y
    if y(i)-p_r*h_r < sm
        u_r(i) = -1000000000;
    else
        u_r(i) = (y(i)-p_r*h_r)^(1-alpha)*(h_r-h_min)^alpha;
    end
end
 

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Find steady state pw, w_bar, qr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SS = @(x) flow_difference_baseline(x, sm,sm1,sm2,gamma,u_p,u_r,u_m,u_s,u_p_s,u_r_s,u_m_s,u_s_s,g_tran,g_tran_s,g_mat,y,y_s,h_m,h_m_s,k_p,k_r,k_m,num_y_s,num_y,max_w,y_low);

options = optimset('Display',disp_op,'TolFun',sm3,'TolX',sm3,'MaxFunEvals',200);
disp('-----------------------------------------------------------------------')
disp('solve model')
disp('-----------------------------------------------------------------------')
[xx,ff,exitflag] = fsolve(SS,x_guess,options);

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(2) = xx(2) +  0.01;
       
      [xx,ff,exitflag] = fsolve(SS,xx,options);
end

[~,g_p,g_r,g_m,g_r_w,g_m_w,policy_pm,policy_prm,policy_pms,policy_prms,policy_rms,policy_ms,policy_rm,v_p,v_r,v_m]= SS(xx);

xx(1) = min(max_w,max(1,xx(1)));
w_bar = floor(xx(1));
pw = 1-(xx(1)-w_bar);
qr = xx(2);
qr = min(1,max(0,qr));

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Simulate moment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%w
disp('-----------------------------------------------------------------------')
disp('simulate')
disp('-----------------------------------------------------------------------')
yt = zeros(n_sim,n_t); % income
dt = zeros(n_sim,n_t); % previous choice: 1:public, 2:regulated, 3:unregulated
ht = zeros(n_sim,n_t); % previous housing size, relavant if dt = 3
wt = zeros(n_sim,n_t); % waiting time: 0,...,w_bar+1, not relavant if dt = 1
lt = zeros(n_sim,n_t); % length of stay
ayt = zeros(n_sim,n_t);
qt = qt_draw <= qr;
pt = pt_draw <= pw;

% initialize

ct = 0;
% (2) regulated housing
for j=0:w_bar+1
    
    num_g = round(n_sim*k_r*g_r_w(j+1));
    
    for i= ct+1:ct+num_g
    
    yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_r(:,j+1)))+1;
    dt(i,1) = 2;
    wt(i,1) = j;
    end
    ct = ct + num_g;    
end

% unregulated housing
g_m_h_w = zeros(num_y, w_bar+2);
g_m_y = zeros(num_y,num_y,w_bar+2);

for j=0:w_bar+1
    for i_h=1:num_y
        g_m_h_w(i_h,j+1) = sum(g_m(:,i_h,j+1))*g_m_w(j+1); % joint density
        temp = sum(g_m(:,i_h,j+1));
        g_m_y(:,i_h,j+1) = g_m(:,i_h,j+1)/temp;            % g_m(y|h,w)
        num_g = round(n_sim*k_m*g_m_h_w(i_h,j+1));
        
        for i= ct+1:min(n_sim-1,ct+num_g)
            yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_m_y(:,i_h,j+1)))+1;
            dt(i,1) = 3;
            ht(i,1) = i_h;
            wt(i,1) = j;
        end
        ct = ct + num_g;    
    end
end

ct = min(n_sim-1,ct);

% (1) public housing
for i = ct+1:n_sim
    yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_p))+1;
    dt(i,1) = 1;
end

yt(:,1) = min(num_y, yt(:,1)); 
yt(:,1) = max(1, yt(:,1));
ht(:,1) = max(1, ht(:,1));

% forward simulation
for t = 1:n_t-1
    for i=1:n_sim
        
         % with public housing options and no regulated housing option : policy_pm(yt)          
         if (dt(i,t) == 1 && qt(i,t)==0) 
             
                if policy_pm(yt(i,t)) == 1
                    dt(i,t+1) = 1;
                else
                    dt(i,t+1) = 3;
                    wt(i,t+1) = 0;
                end                           
         
         % with public and regulated housing options : policy_prm(yt,dt)         
         elseif (dt(i,t) == 1 && qt(i,t)==1  ) ... 
             || (dt(i,t) == 2 && wt(i,t) == w_bar   && pt(i,t)==1  && y(yt(i,t)) <= y_low) ...
             || (dt(i,t) == 2 && wt(i,t) == w_bar+1 && y(yt(i,t)) <= y_low)
             
                 if policy_prm(yt(i,t),dt(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_prm(yt(i,t),dt(i,t)) == 2
                    dt(i,t+1) = 2;
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    wt(i,t+1) = 0;
                 end
                 
         % policy_pms(yt,ht): dt= 3 by construction         
         elseif (dt(i,t) == 3 && wt(i,t) == w_bar   && pt(i,t)==1  && qt(i,t)==0 && y(yt(i,t)) <= y_low) ...
             || (dt(i,t) == 3 && wt(i,t) == w_bar+1 && qt(i,t)==0 && y(yt(i,t)) <= y_low)
             
                 if policy_pms(yt(i,t),ht(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_pms(yt(i,t),ht(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                    wt(i,t+1) = 0;
                 end    
                 
                 
         % policy_prms(yt,ht): dt= 3 by construction         
         elseif (dt(i,t) == 3 && wt(i,t) == w_bar   && pt(i,t)==1  && qt(i,t)==1 && y(yt(i,t)) <= y_low) ...
             || (dt(i,t) == 3 && wt(i,t) == w_bar+1 && qt(i,t)==1 && y(yt(i,t)) <= y_low)
             
                 if policy_prms(yt(i,t),ht(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_prms(yt(i,t),ht(i,t)) == 2
                    dt(i,t+1) = 2;
                    wt(i,t+1) = 0;
                 elseif policy_prms(yt(i,t),ht(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                    wt(i,t+1) = 0;
                 end     
                 
         % policy_rms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==1 && y(yt(i,t)) <= y_low) 
             
                 if policy_rms(yt(i,t),ht(i,t),wt(i,t)+1) == 2
                    dt(i,t+1) = 2;
                 elseif policy_rms(yt(i,t),ht(i,t),wt(i,t)+1) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = wt(i,t)+1;
                 
         % policy_ms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==0 && y(yt(i,t)) <= y_low) 
             
                 if policy_ms(yt(i,t),ht(i,t),wt(i,t)+1) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = wt(i,t)+1;      
                 
         % policy_rm(yt,dt,wt)   
         elseif  (dt(i,t) == 2  && y(yt(i,t)) <= y_low)
             
                 if policy_rm(yt(i,t),dt(i,t),wt(i,t)+1) == 2
                    dt(i,t+1) = 2;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 end                        
                 wt(i,t+1) = wt(i,t)+1;       
                 
                 
         % policy_rms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==1 && y(yt(i,t)) > y_low) 
             
                 if policy_rms(yt(i,t),ht(i,t),1) == 2
                    dt(i,t+1) = 2;
                 elseif policy_rms(yt(i,t),ht(i,t),1) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = 0;
                 
         % policy_ms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==0 && y(yt(i,t)) > y_low) 
             
                 if policy_ms(yt(i,t),ht(i,t),1) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = 0;     
                 
         % policy_rm(yt,dt,wt)   
         elseif  (dt(i,t) == 2  && y(yt(i,t)) > y_low)
             
                 if policy_rm(yt(i,t),dt(i,t),1) == 2
                    dt(i,t+1) = 2;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 end                        
                 wt(i,t+1) = 0;                   
               
         end         

        % update duration
         
         if (dt(i,t) <= 2 && dt(i,t+1) == dt(i,t)) || (dt(i,t) == 3 && dt(i,t+1) == dt(i,t) && ht(i,t+1) == ht(i,t))
             lt(i,t+1) = lt(i,t)+1;
         else
             lt(i,t+1) = 0;
         end
         
         % update income
         
         yt(i,t+1) = sum(yt_draw(i,t) >= y_cdf(yt(i,t),:))+1;   
               
    end  
    yt(:,t+1) = min(num_y, yt(:,t+1));  
    yt(:,t+1) = max(1, yt(:,t+1));
    ht(:,t+1) = max(1,ht(:,t+1));
    
end
%% calculate moment

    ayt(:,:) = y(yt(:,:));
    model = zeros(13,1);    
    dt_s = dt(:,n_burn:n_t);
    lt_s= lt(:,n_burn:n_t);
    ayt_s = ayt(:,n_burn-1:n_t-1);
    %ayt_s = ayt(:,n_burn:n_t);
    ht_s  = (h_min + alpha*(ayt_s-h_min*p_m));

    model(1) = mean(ayt_s(dt_s==1));
    model(2) = mean(ayt_s(dt_s==2));
    model(3) = mean(ayt_s(dt_s==3));
    model(4) = mean(lt_s(dt_s==1));
    model(5) = mean(lt_s(dt_s==2));
    model(6) = mean(lt_s(dt_s==3));
    model(7) = std(ayt_s(dt_s==1));
    model(8) = h_r/12*p_r;
    model(9) = mean(ht_s(dt_s==3 & ayt_s>=75000 & ayt_s<100000))/12;
    model(10)= mean(ht_s(dt_s==3 & ayt_s>=100000 & ayt_s<125000))/12;
    model(11)= mean(ht_s(dt_s==3 & ayt_s>=125000 & ayt_s<150000))/12;
    model(12) = sum(sum((ayt_s>=y_low).*(dt_s==1)))/sum(sum((dt_s==1)));
    model(13) = mean(ayt_s(dt_s==1 & ayt_s>=y_low))/y_low-1;
    

  
 
   
format longG  
disp('table 3')
disp([alpha h_min gamma mu sigma rho h h_r]')


disp('table 4')
disp([w_bar pw qr]')


disp('table 5')
disp([data(4:6,1) model(4:6,1) data(1:3,1) model(1:3,1)])
disp([data(8:11,1) model(8:11,1)])


figure('name','policy')
plot(y, policy_prm(:,1),'b--','LineWidth',2)
hold on
plot(y, policy_pm,'r-.','LineWidth',2)
line([(y_low) (y_low)], [1 3])
xlabel ('income')
hleg = legend('with an offer to regulated market','without an offer to regulated market');
set(hleg, 'Box','off','Location','NorthWest','Interpreter','Tex')
ax = gca;
ax.YTick = [1 2 3];
axis([0 200000  1 3])  
set(gca,'FontSize',16)


width = log_y(2) - log_y(1);
g_p_level = g_p./(exp(log_y+width/2)-exp(log_y-width/2));
g_r_level = zeros(num_y,10);
g_m_level = zeros(num_y,10);
for i=1:10
    g_r_level(:,i) = g_r(:,i)./(exp(log_y+width/2)-exp(log_y-width/2));
    g_m_level(:,i) = sum(g_m(:,:,i),2)./(exp(log_y+width/2)-exp(log_y-width/2));
end

figure('name','distribution')
plot(y,g_p_level,'b--','LineWidth',1)
hold on
plot(y,g_r_level(:,1),'r-','LineWidth',1)
hold on
plot(y,g_r_level(:,6),'k-.','LineWidth',1)
hold on
line([y_low y_low], [0 0.00015])
hleg = legend('g_p(y)','g_r(y|0)','g_r(y|5)');
set(hleg, 'Box','off','Location','NorthEast','Interpreter','Tex')
xlabel ('income')
ylabel ('density')
axis([0 200000  0 0.00015])
set(gca,'FontSize',16)

figure('name','distribution2')
plot(y,g_p_level,'b--','LineWidth',1)
hold on
plot(y,g_m_level(:,1),'r-','LineWidth',1)
hold on
plot(y,g_m_level(:,6),'k-.','LineWidth',1)
hold on
line([y_low y_low], [0 0.00015])
hleg = legend('g_p(y)','g_m(y|0)','g_m(y|5)');
set(hleg, 'Box','off','Location','NorthEast','Interpreter','Tex')
xlabel ('income')
ylabel ('density')
axis([0 200000  0 0.00015])   
set(gca,'FontSize',16)






