%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0. Basic settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; 
clc;
disp_op    = 'off';  % show fsolve iteration
beta    = 0.95;
sm      = 1d-7;      % small number for log or devide
sm1     = 1d-7;      % convergence criterion for value function iteration
sm2     = 1d-7;      % convergence criterion for SS distribution iteration
sm3     = 1d-8;      % convergence criterion for fsolve
num_y_s = 50;        % number of income grids
num_y   = 100;       % number of income grids
num_r   = 2;
max_w   = 50;        % maximum value of waiting list
n_burn  = 50;
 
x_guess =  [   19.81
              0.11
              0.26];  

y_low   = 52367.227;
tau     = 0.3;       % tax rate in public housing
p_m       = 1;         % market rent
p_r(1)  = 1-0.5125977; % price of regulated housing type 1
p_r(2)  = 1-0.5125977; % price of regulated housing type 2
k_p     = 0.0990;
k_r     = [0.2863 0.2857]; % fraction of regulated 1 and 2
k_m     = 1-k_p-sum(k_r);  % fraction of market
rho   = 0.6661;
% generate randome sample 
n_sim         = 5000;      % number of simulation sample
n_t           = 100;       % length of simulation
rng(10) % random number seed
qt_draw = rand(n_sim,n_t);
pt_draw = rand(n_sim,n_t);
yt_draw = rand(n_sim,n_t);
% read data and parameter values
data = xlsread('moment_1hh.xlsx')';
load('test_1hh.mat')
x=xx;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Setting up
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read input parameters
h     = x(1);
h_r   = x(2:3);
mu    = x(4);
sigma = x(5);
alpha = x(6);
gamma = x(7);
h_min = x(8);

y_min = h_min*p_m; % minimum income

% algorithm stabilization tricks
sigma = max(sm,sigma);
mu = max(sm,mu);

% Tauchen small grids
m =4;                                                  % coverage m standard deviation
[log_y_s,g_tran_s] = tauchen(num_y_s,mu,rho,sigma,m);  % (log income, transition matrix)
y_s = exp(log_y_s)+y_min;                              % income level

% define flow utility function
u_p_s = ((1-tau)*y_s).^(1-alpha).*(h-h_min).^alpha;            % public
u_r_s = zeros(num_y_s,num_r);                                            % regulated
u_m_s = alpha^alpha*(1-alpha)^(1-alpha)*(y_s-h_min*p_m)/p_m^alpha; % mover
h_m_s = alpha*(y_s-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s_s = zeros(num_y_s,num_y_s);                                      % stayer

for i=1:num_y_s
    for j=1:num_y_s
        if y_s(i)-p_m*h_m_s(j)< sm
            u_s_s(i,j) = -1000000000;
        else
            u_s_s(i,j) = (y_s(i)-p_m*h_m_s(j))^(1-alpha)*(h_m_s(j)-h_min)^alpha;
        end
    end
end

for i_r=1:num_r
    for i_y=1:num_y_s
        if y_s(i_y)-p_r(i_r)*h_r(i_r) < sm
            u_r_s(i_y,i_r) = -1000000000;
        else
            u_r_s(i_y,i_r) = (y_s(i_y)-p_r(i_r)*h_r(i_r))^(1-alpha)*(h_r(i_r)-h_min)^alpha;
        end
    end
end
 
%Tauchen approximation of AR1
m =4;             % coverage m standard deviation
[log_y,g_tran] = tauchen(num_y,mu,rho,sigma,m); % (log income, transition matrix)
y = exp(log_y)+y_min;   % income level

y_cdf = zeros(num_y,num_y); % CDF of income process
for i=1:num_y
    y_cdf(:,i) = sum(g_tran(:,1:i),2);
end

g_mat = g_tran';

% define flow utility function
u_p = ((1-tau)*y).^(1-alpha).*(h-h_min).^alpha;            % public
u_r = zeros(num_y,num_r);                                            % regulated
u_m = alpha^alpha*(1-alpha)^(1-alpha)*(y-h_min*p_m)/p_m^alpha; % mover
h_m = alpha*(y-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s = zeros(num_y,num_y);                                      % stayer

for i=1:num_y
    for j=1:num_y
        if y(i)-p_m*h_m(j)< sm
            u_s(i,j) = -1000000000;
        else
            u_s(i,j) = (y(i)-p_m*h_m(j))^(1-alpha)*(h_m(j)-h_min)^alpha;
        end
    end
end

for i_r = 1:num_r
    for i_y=1:num_y
        if y(i_y)-p_r(i_r)*h_r(i_r)< sm
            u_r(i_y,i_r) = -1000000000;
        else
            u_r(i_y,i_r) = (y(i_y)-p_r(i_r)*h_r(i_r))^(1-alpha)*(h_r(i_r)-h_min)^alpha;
        end
    end
end
 

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Find steady state pw, w_bar, qr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SS = @(x) flow_difference_1hh(x, sm, sm1, sm2, gamma,u_p,u_r,u_m,u_s,u_p_s,u_r_s,u_m_s,u_s_s,g_tran,g_tran_s,g_mat,y,y_s,h_m,h_m_s,k_p,k_r,k_m,num_y_s,num_y,num_r,max_w,y_low);

options = optimset('Display',disp_op,'TolFun',sm3,'TolX',sm3,'MaxFunEvals',200);

disp('-----------------------------------------------------------------------')
disp('solve model')
disp('-----------------------------------------------------------------------')
[xx,ff,exitflag] = fsolve(SS,x_guess,options);

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(2) = xx(2) +  0.01;
       
      [xx,ff,exitflag] = fsolve(SS,xx,options);
end

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(3) = xx(3) +  0.01;
       
      [xx,ff,exitflag] = fsolve(SS,xx,options);
end

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(2:3) = xx(2:3) +  0.01;
       
      [xx,ff,exitflag] = fsolve(SS,xx,options);
end

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(1) = xx(1) -  0.5;
       
      [xx,ff,exitflag] = fsolve(SS,xx,options);
end

[~,g_p,g_r,g_m,g_r_w,g_m_w,policy_pm,policy_prm,policy_pms,policy_prms,policy_rms,policy_ms,policy_rm,v_p,v_r,v_m]= SS(xx);

xx(1) = min(max_w,max(1,xx(1)));
w_bar = floor(xx(1));
pw = 1-(xx(1)-w_bar);
qr=xx(2:num_r+1);
qr = min(1,max(0,qr));

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Simulate moment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%w
disp('-----------------------------------------------------------------------')
disp('simulate')
disp('-----------------------------------------------------------------------')
yt = zeros(n_sim,n_t); % income
dt = zeros(n_sim,n_t); % previous choice: 1:public, 2:regulated, 3:unregulated
ht = zeros(n_sim,n_t); % previous housing size, relavant if dt = 3
wt = zeros(n_sim,n_t); % waiting time: 0,...,w_bar+1, not relavant if dt = 1
lt = zeros(n_sim,n_t); % length of stay
ayt = zeros(n_sim,n_t);
qt = 1*(qt_draw <= qr(1)) + 2*(qt_draw > qr(1)).*(qt_draw <= sum(qr));
pt = pt_draw <= pw;

% initialize


ct = 0;
% (2) regulated housing
for i_r = 1:num_r
    for j=0:w_bar+1

        num_g = round(n_sim*k_r(i_r)*g_r_w(j+1,i_r));

        for i= ct+1:ct+num_g

        yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_r(:,j+1,i_r)))+1;
        dt(i,1) = 2;
        wt(i,1) = j;
        qt(i,1) = i_r;
        end
        ct = ct + num_g;    
    end
end

% unregulated housing

g_m_h_w = zeros(num_y, w_bar+2);
g_m_y = zeros(num_y,num_y,w_bar+2);

for j=0:w_bar+1
    for i_h=1:num_y
        g_m_h_w(i_h,j+1) = sum(g_m(:,i_h,j+1))*g_m_w(j+1);
        temp = sum(g_m(:,i_h,j+1));        
        g_m_y(:,i_h,j+1) = g_m(:,i_h,j+1)/temp;
        num_g = round(n_sim*k_m*g_m_h_w(i_h,j+1));
        num_g = max(0,num_g);       
        
        if num_g>=1
        
        for i= ct+1:ct+num_g
            if i<=n_sim
                yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_m_y(:,i_h,j+1)))+1;
                dt(i,1) = 3;
                ht(i,1) = i_h;
                wt(i,1) = j;
            end
        end
        end
        ct = ct + num_g;    
    end
end
ct = min(n_sim-1,ct);

% (1) public housing
for i = ct+1:n_sim
    yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_p))+1;
    dt(i,1) = 1;
end
     
yt(:,1) = min(num_y, yt(:,1)); 
yt(:,1) = max(1, yt(:,1));
ht(:,1) = max(1, ht(:,1));
 
% forward simulation
for t = 1:n_t-1
    for i=1:n_sim
        
         % with public housing options and no regulated housing option : policy_pm(yt)          
         if    (dt(i,t) == 1 && qt(i,t)==0  ) 
             
                if policy_pm(yt(i,t)) == 1
                    dt(i,t+1) = 1;
                else
                    dt(i,t+1) = 3;
                    wt(i,t+1) = 0;
                end                           
         
         % with public and regulated housing options : policy_prm(yt,dt): h1     
         elseif (dt(i,t) == 1 && qt(i,t)>=1  ) ... 
             || (dt(i,t) == 2 && wt(i,t) == w_bar   && pt(i,t)==1  && y(yt(i,t)) <= y_low) ...
             || (dt(i,t) == 2 && wt(i,t) == w_bar+1 && y(yt(i,t)) <= y_low)
             
                 if policy_prm(yt(i,t),dt(i,t),qt(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_prm(yt(i,t),dt(i,t),qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    wt(i,t+1) = 0;
                    qt(i,t+1) = qt(i,t);
                 else
                    dt(i,t+1) = 3;
                    wt(i,t+1) = 0;
                 end
                 
                 
         % policy_pms(yt,ht): dt= 3 by construction         
         elseif (dt(i,t) == 3 && wt(i,t) == w_bar   && pt(i,t)==1  && qt(i,t)==0 && y(yt(i,t)) <= y_low) ...
             || (dt(i,t) == 3 && wt(i,t) == w_bar+1 && qt(i,t)==0 && y(yt(i,t)) <= y_low)
             
                 if policy_pms(yt(i,t),ht(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_pms(yt(i,t),ht(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                    wt(i,t+1) = 0;
                 end    
                 
                 
         % policy_prms(yt,ht): dt= 3 by construction         
         elseif (dt(i,t) == 3 && wt(i,t) == w_bar   && pt(i,t)==1  && qt(i,t)>=1 && y(yt(i,t)) <= y_low) ...
             || (dt(i,t) == 3 && wt(i,t) == w_bar+1 && qt(i,t)>=1 && y(yt(i,t)) <= y_low)
             
                 if policy_prms(yt(i,t),ht(i,t),qt(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_prms(yt(i,t),ht(i,t),qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    wt(i,t+1) = 0;
                    qt(i,t+1) = qt(i,t);
                 elseif policy_prms(yt(i,t),ht(i,t),qt(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                    wt(i,t+1) = 0;
                 end     
                 
         % policy_rms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)>=1 && y(yt(i,t)) <= y_low) 
             
                 if policy_rms(yt(i,t),ht(i,t),wt(i,t)+1,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1)= qt(i,t);
                 elseif policy_rms(yt(i,t),ht(i,t),wt(i,t)+1,qt(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = wt(i,t)+1;
                 
         % policy_ms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==0 && y(yt(i,t)) <= y_low) 
             
                 if policy_ms(yt(i,t),ht(i,t),wt(i,t)+1) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = wt(i,t)+1;      
                 
         % policy_rm(yt,dt,wt)   
         elseif  (dt(i,t) == 2  && y(yt(i,t)) <= y_low)
                 if policy_rm(yt(i,t),dt(i,t),wt(i,t)+1,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1) = qt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 end                        
                 wt(i,t+1) = wt(i,t)+1;       
                 
                 
         % policy_rms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)>=1 && y(yt(i,t)) > y_low) 
             
                 if policy_rms(yt(i,t),ht(i,t),1,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1) = qt(i,t);
                 elseif policy_rms(yt(i,t),ht(i,t),1,qt(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = 0;
                 
         % policy_ms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==0 && y(yt(i,t)) > y_low) 
             
                 if policy_ms(yt(i,t),ht(i,t),1) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = 0;     
                 
         % policy_rm(yt,dt,wt)   
         elseif  (dt(i,t) == 2  && y(yt(i,t)) > y_low) 
             
                 if policy_rm(yt(i,t),dt(i,t),1,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1) = qt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 end                        
                 wt(i,t+1) = 0;                   
               
         end         

        % update duration
         
         if (dt(i,t) <= 2 && dt(i,t+1) == dt(i,t)) || (dt(i,t) == 3 && dt(i,t+1) == dt(i,t) && ht(i,t+1) == ht(i,t))
             lt(i,t+1) = lt(i,t)+1;
         else
             lt(i,t+1) = 0;
         end
         
         % update income
         
         yt(i,t+1) = sum(yt_draw(i,t) >= y_cdf(yt(i,t),:))+1;   
               
    end  
    yt(:,t+1) = min(num_y, yt(:,t+1));  
    yt(:,t+1) = max(1, yt(:,t+1));
    ht(:,t+1) = max(1,ht(:,t+1));
    
end
%% calculate moment

ayt(:,:) = y(yt(:,:));
%toc;tic
    model = zeros(16,1);    
    dt_s = dt(:,n_burn:n_t);
    qt_s = qt(:,n_burn:n_t);
    lt_s= lt(:,n_burn:n_t);
    ayt_s = ayt(:,n_burn-1:n_t-1);
    %ayt_s = ayt(:,n_burn:n_t);
    ht_s  = (h_min + alpha*(ayt_s-h_min*p_m));

    model(1) = mean(ayt_s(dt_s==1));
    model(2) = mean(ayt_s(dt_s==2 & qt_s==1));
    model(3) = mean(ayt_s(dt_s==2 & qt_s==2));
    model(4) = mean(ayt_s(dt_s==3));
    model(5) = mean(lt_s(dt_s==1));
    model(6) = mean(lt_s(dt_s==2 & qt_s==1));
    model(7) = mean(lt_s(dt_s==2 & qt_s==2));
    model(8) = mean(lt_s(dt_s==3));
    model(9) = std(ayt_s(dt_s==1));
    model(10) = h_r(1)/12*p_r(1);
    model(11) = h_r(2)/12*p_r(2);
    model(12)=mean(ht_s(dt_s==3 & ayt_s>=75000 & ayt_s<100000))/12;
    model(13)=mean(ht_s(dt_s==3 & ayt_s>=100000 & ayt_s<125000))/12;
    model(14)=mean(ht_s(dt_s==3 & ayt_s>=125000 & ayt_s<150000))/12;
    model(15) = sum(sum((ayt_s>=y_low).*(dt_s==1)))/sum(sum((dt_s==1)));
    model(16) = mean(ayt_s(dt_s==1 & ayt_s>=y_low))/y_low-1; 
    


  
 
format longG  
disp('table 3')
disp([alpha h_min gamma mu sigma rho h h_r']')


disp('table 4')
disp([w_bar pw qr']')


disp('table 5')
disp([data(5:8,1) model(5:8,1) data(1:4,1) model(1:4,1)])
disp([data(10:14,1) model(10:14,1)])


w_gain_m_r = zeros(num_y,num_r);
v_m_marginal = sum(v_m(:,:,10).*g_m(:,:,10),2)./sum(g_m(:,:,10),2);

n_start = 40;
for j = 1:num_r
    for i=n_start:num_y
        sqr1 = @(x) interp1(log_y,v_m_marginal,x)-v_r(i,10,j);

        w_gain_m_r(i,j) = fsolve(sqr1,log_y(i),options);
    end
w_gain_m_r(:,j) = exp(w_gain_m_r(:,j)) - exp(log_y);    
end
w_gain_m_r(:,:) = w_gain_m_r(:,:)/(1-beta*rho);

figure
plot(y(n_start:end),w_gain_m_r(n_start:end,1),'b-','LineWidth',3)
hold on
plot(y(n_start:end),w_gain_m_r(n_start:end,2),'k-.','LineWidth',3)
hleg = legend('low quality','high quality');
set(hleg, 'Box','off','Location','SouthEast','Interpreter','Tex')
xlabel('income')
ylabel('compensating variation')
axis([0 200000 -20000 70000])
set(gca,'FontSize',16)










