%% basic settings
clear all; 
clc;
%%
load('test_1hh.mat')
x=xx;

beta    = 0.95;
sm      = 1d-7;      % small number for log or devide
sm1     = 1d-7;      % convergence criterion for value function iteration
sm2     = 1d-7;      % convergence criterion for SS distribution iteration
sm3     = 1d-8;      % convergence criterion for fsolve
num_y_s = 50;        % number of income grids
num_y   = 100;       % number of income grids
num_r   = 2;
max_w   = 50;        % maximum value of waiting list
n_burn  = 50;
 
x_guess =  [   19.81
              0.11
              0.26];  

y_low   = 52367.227;
tau     = 0.3;       % tax rate in public housing
p_m       = 1;         % market rent
p_r(1)  = 1-0.5125977; % price of regulated housing type 1
p_r(2)  = 1-0.5125977; % price of regulated housing type 2
k_p     = 0.0990;
k_r     = [0.2863 0.2857]; % fraction of regulated 1 and 2
k_m     = 1-k_p-sum(k_r);  % fraction of market
rho   = 0.6661;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Setting up
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read input parameters
h     = x(1);
h_r   = x(2:3);
mu    = x(4);
sigma = x(5);
alpha = x(6);
gamma = x(7);
h_min = x(8);

y_min = h_min*p_m; % minimum income

% algorithm stabilization tricks
sigma = max(sm,sigma);
mu = max(sm,mu);

% Tauchen small grids
m =4;                                                  % coverage m standard deviation
[log_y_s,g_tran_s] = tauchen(num_y_s,mu,rho,sigma,m);  % (log income, transition matrix)
y_s = exp(log_y_s)+y_min;                              % income level

% define flow utility function
u_p_s = ((1-tau)*y_s).^(1-alpha).*(h-h_min).^alpha;            % public
u_r_s = zeros(num_y_s,num_r);                                            % regulated
u_m_s = alpha^alpha*(1-alpha)^(1-alpha)*(y_s-h_min*p_m)/p_m^alpha; % mover
h_m_s = alpha*(y_s-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s_s = zeros(num_y_s,num_y_s);                                      % stayer

for i=1:num_y_s
    for j=1:num_y_s
        if y_s(i)-p_m*h_m_s(j)< sm
            u_s_s(i,j) = -1000000000;
        else
            u_s_s(i,j) = (y_s(i)-p_m*h_m_s(j))^(1-alpha)*(h_m_s(j)-h_min)^alpha;
        end
    end
end

for i_r=1:num_r
    for i_y=1:num_y_s
        if y_s(i_y)-p_r(i_r)*h_r(i_r) < sm
            u_r_s(i_y,i_r) = -1000000000;
        else
            u_r_s(i_y,i_r) = (y_s(i_y)-p_r(i_r)*h_r(i_r))^(1-alpha)*(h_r(i_r)-h_min)^alpha;
        end
    end
end
 
%Tauchen approximation of AR1
m =4;             % coverage m standard deviation
[log_y,g_tran] = tauchen(num_y,mu,rho,sigma,m); % (log income, transition matrix)
y = exp(log_y)+y_min;   % income level

y_cdf = zeros(num_y,num_y); % CDF of income process
for i=1:num_y
    y_cdf(:,i) = sum(g_tran(:,1:i),2);
end

g_mat = g_tran';

% define flow utility function
u_p = ((1-tau)*y).^(1-alpha).*(h-h_min).^alpha;            % public
u_r = zeros(num_y,num_r);                                            % regulated
u_m = alpha^alpha*(1-alpha)^(1-alpha)*(y-h_min*p_m)/p_m^alpha; % mover
h_m = alpha*(y-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s = zeros(num_y,num_y);                                      % stayer

for i=1:num_y
    for j=1:num_y
        if y(i)-p_m*h_m(j)< sm
            u_s(i,j) = -1000000000;
        else
            u_s(i,j) = (y(i)-p_m*h_m(j))^(1-alpha)*(h_m(j)-h_min)^alpha;
        end
    end
end

for i_r = 1:num_r
    for i_y=1:num_y
        if y(i_y)-p_r(i_r)*h_r(i_r)< sm
            u_r(i_y,i_r) = -1000000000;
        else
            u_r(i_y,i_r) = (y(i_y)-p_r(i_r)*h_r(i_r))^(1-alpha)*(h_r(i_r)-h_min)^alpha;
        end
    end
end
 

n_grid = 7;
kr_grid = zeros(n_grid,1);
welfare_p = zeros(n_grid,num_y);
welfare_r = zeros(n_grid,num_y,num_r);
welfare_m = zeros(n_grid,num_y);
welfare_all = zeros(n_grid,num_y);
solution_grid = zeros(n_grid,4);


for i = 1:n_grid

k_p     = 0.0990;     % fraction of public
k_r     = [0.2863 0.2857]+(i-1)*0.005; % fraction of regulated 1 and 2
k_m     = 1-k_p-sum(k_r);  % fraction of market
kr_grid(i,:) = sum(k_r);

disp('------------')
disp('solve the model when the share of regulated housing is ')
disp(sum(k_r))
disp('------------')


if i>1 && i<=5
x_guess(1) = xx(1) - 0.25;
x_guess(2) = xx(2) + 0.01;
x_guess(3) = xx(3) + 0.01;
elseif i>5 && i<=10
x_guess(1) = xx(1) - 0.5;
x_guess(2) = xx(2) + 0.02;
x_guess(3) = xx(3) + 0.03;
end

SS = @(x) flow_difference_1hh(x, sm, sm1, sm2, gamma,u_p,u_r,u_m,u_s,u_p_s,u_r_s,u_m_s,u_s_s,g_tran,g_tran_s,g_mat,y,y_s,h_m,h_m_s,k_p,k_r,k_m,num_y_s,num_y,num_r,max_w,y_low);

options = optimset('Display','off','TolFun',sm3,'TolX',sm3,'MaxFunEvals',200);

[xx,ff,exitflag] = fsolve(SS,x_guess,options);

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(2:3) = xx(2:3) +  0.01;
       
      [xx,~,~] = fsolve(SS,xx,options);
end

[~,g_p,g_r,g_m,g_r_w,g_m_w,~,~,~,~,~,~,~,vv_p,vv_r,vv_m]= SS(xx);

xx(1) = min(max_w,max(1,xx(1)));
w_bar = floor(xx(1));
num_w = w_bar+3;

g_p_joint = zeros(num_y,1);
g_r_joint = zeros(num_y,num_w,num_r);
g_m_joint = zeros(num_y,num_y,num_w);


g_p_joint(:) = k_p*g_p(:);
for j=1:w_bar+2
    g_m_joint(:,:,j) = k_m*g_m_w(j)*g_m(:,:,j);
    for r=1:num_r
    g_r_joint(:,j,r) = k_r(r)*g_r_w(j,r)*g_r(:,j,r);
    end
end

welfare_p(i,:) = vv_p.*g_p_joint./g_p_joint;
welfare_r(i,:,:) = sum(vv_r.*g_r_joint,2)./sum(g_r_joint,2);
welfare_m(i,:) = sum(sum(vv_m.*g_m_joint,3),2)./sum(sum(g_m_joint,3),2);


welfare_all(i,:) = (vv_p.*g_p_joint + squeeze(sum(sum(vv_r.*g_r_joint,3),2)) + squeeze(sum(sum(vv_m.*g_m_joint,3),2)))...
    ./(g_p_joint+squeeze(sum(sum(g_r_joint,3),2))+squeeze(sum(sum(g_m_joint,3),2)));

solution_grid(i,1) = xx(1);
solution_grid(i,2) = xx(2);
solution_grid(i,3) = xx(3);
end

figure
plot(kr_grid,solution_grid(:,2),'b-','LineWidth',4);
hold on
plot(kr_grid,solution_grid(:,3),'k-.','LineWidth',4);
hleg = legend('low','high');
set(hleg, 'Box','off','Location','NorthWest','Interpreter','Tex')
xlabel('supply of regulated housing')
set(gca,'FontSize',16)


%% calculate welfare gains
disp('------------')
disp('calulate welfare gains ')
disp('------------')

w_gain = zeros(5,num_y);
w_gain_t = zeros(n_grid,num_y);

options = optimoptions('fsolve','Display','off');

for i=1:num_y-10
    sqr1 = @(x) interp1(y(:),welfare_p(1,:),x)-welfare_p(n_grid,i);
    sqr2 = @(x) interp1(y(:),welfare_r(1,:,1),x)-welfare_r(n_grid,i,1);
    sqr3 = @(x) interp1(y(:),welfare_r(1,:,2),x)-welfare_r(n_grid,i,2);
    sqr4 = @(x) interp1(y(:),welfare_m(1,:),x)-welfare_m(n_grid,i);
    sqr5 = @(x) interp1(y(:),welfare_all(1,:),x)-welfare_all(n_grid,i);

    w_gain(1,i) = fsolve(sqr1,y(i),options)-y(i);
    w_gain(2,i) = fsolve(sqr2,y(i),options)-y(i);
    w_gain(3,i) = fsolve(sqr3,y(i),options)-y(i);
    w_gain(4,i) = fsolve(sqr4,y(i),options)-y(i);
    w_gain(5,i) = fsolve(sqr5,y(i),options)-y(i);
end
    
for i=num_y-10+1:num_y
    sqr1 = @(x) interp1(y(:),welfare_p(n_grid,:),x)-welfare_p(1,i);
    sqr2 = @(x) interp1(y(:),welfare_r(n_grid,:,1),x)-welfare_r(1,i,1);
    sqr3 = @(x) interp1(y(:),welfare_r(n_grid,:,2),x)-welfare_r(1,i,2);
    sqr4 = @(x) interp1(y(:),welfare_m(n_grid,:),x)-welfare_m(1,i);
    sqr5 = @(x) interp1(y(:),welfare_all(n_grid,:),x)-welfare_all(1,i);

    w_gain(1,i) = y(i)- fsolve(sqr1,y(i),options);
    w_gain(2,i) = y(i)- fsolve(sqr2,y(i),options);
    w_gain(3,i) = y(i)- fsolve(sqr3,y(i),options);
    w_gain(4,i) = y(i)- fsolve(sqr4,y(i),options);
    w_gain(5,i) = y(i)- fsolve(sqr5,y(i),options);
end

for k=2:n_grid
        for i=1:num_y-10
            sqr6 = @(x) interp1(y(:),welfare_all(1,:),x)-welfare_all(k,i);
            w_gain_t(k,i) = fsolve(sqr6,y(i),options) - y(i);
        end
end

for k=2:n_grid
        for i=num_y-10+1:num_y
            sqr6 = @(x) interp1(y(:),welfare_all(k,:),x)-welfare_all(1,i);
            w_gain_t(k,i) = y(i) - fsolve(sqr6,y(i),options);
        end
end


w_gain(:,:) = w_gain(:,:) /(1-beta*rho);


w_gain_t(:,:) = w_gain_t(:,:)/(1-beta*rho);


w_gain_t_sum = zeros(n_grid,1);

for k=2:n_grid
        w_gain_t_sum(k) = sum(w_gain_t(k,:).*mean(g_mat(:,:),2)');

end

figure
plot(kr_grid,w_gain_t_sum,'-','LineWidth',4)
xlabel('supply of regulated housing')
ylabel('compensating variation')
set(gca,'FontSize',16)








