%% This program conducts policy experiment of progressive tax on public housing
clear all; 
clc;
%% settings
disp_op    = 'off';  % show fsolve iteration
num_est       = 200; % maximum number of function evaluation
%% parameters and variables
beta    = 0.95;
rho     = 0.6661;
num_y   = 100;       % number of income grids
num_r   = 2;
max_w   = 50;        % maximum value of waiting list
k_p     = 0.0990;
k_r     = [0.2863 0.2857]; % fraction of regulated 1 and 2
k_m     = 1-k_p-sum(k_r);  % fraction of market

n_grid = 4;
tau_grid  = linspace(0.3,0.20,n_grid);
welfare_p = zeros(n_grid,num_y);
welfare_r = zeros(n_grid,num_y,num_r);
welfare_m = zeros(n_grid,num_y);
welfare_all = zeros(n_grid,num_y);
solution_grid = zeros(n_grid,3);
tau1_grid = zeros(n_grid,1);
tax_grid = zeros(n_grid,num_y);
g_p_grid = zeros(n_grid,num_y);
policy_pm_grid = zeros(n_grid,num_y);
policy_prm_grid = zeros(n_grid,num_y);

%% run experiment

for i = 1:n_grid
    
    tau = tau_grid(i);
    
    disp('-------------------------------------------')
    disp('solve model when tau is')
    disp(tau)
    disp('-------------------------------------------')
    
    if i==1 % baseline of tau = 0.3, find the baseline tax revenus
        rev_data = 0;
        disp_op = 'off';
        Q = @(x) revenue(x,tau,rev_data,disp_op);
        x0 = 0;
        [rev_init,g_p,g_r,g_m,g_r_w,g_m_w,vv_p,vv_r,vv_m,solution,y,g_mat,tax,policy_pm,policy_prm]=Q(x0);
        xx = x0;
    else 
        rev_data = rev_init;
        disp_op = 'off';
        Q = @(x) revenue(x,tau,rev_data,disp_op);
        options = optimset('Display','off','MaxFunEvals',num_est);
        x0 = xx;
        xx = fsolve(Q,x0,options); % find the revenue netural tau1
        Q = @(x) revenue(x,tau,rev_data,disp_op);
        [~,g_p,g_r,g_m,g_r_w,g_m_w,vv_p,vv_r,vv_m,solution,~,~,tax,policy_pm,policy_prm]=Q(xx); % solve the model 
        
        tau1_grid(i,1) = xx;
    end

w_bar = floor(solution(1));
num_w = w_bar+3;

g_p_joint = zeros(num_y,1);
g_r_joint = zeros(num_y,num_w,num_r);
g_m_joint = zeros(num_y,num_y,num_w);


g_p_joint(:) = k_p*g_p(:);
for j=1:w_bar+2
    g_m_joint(:,:,j) = k_m*g_m_w(j)*g_m(:,:,j);
    for r=1:num_r
    g_r_joint(:,j,r) = k_r(r)*g_r_w(j,r)*g_r(:,j,r);
    end
end

welfare_p(i,:) = vv_p.*g_p_joint./g_p_joint;
welfare_r(i,:,:) = sum(vv_r.*g_r_joint,2)./sum(g_r_joint,2);
welfare_m(i,:) = sum(sum(vv_m.*g_m_joint,3),2)./sum(sum(g_m_joint,3),2);


welfare_all(i,:) = (vv_p.*g_p_joint + squeeze(sum(sum(vv_r.*g_r_joint,3),2)) + squeeze(sum(sum(vv_m.*g_m_joint,3),2)))...
    ./(g_p_joint+squeeze(sum(sum(g_r_joint,3),2))+squeeze(sum(sum(g_m_joint,3),2)));

solution_grid(i,:) = solution(:);
tax_grid(i,:) = tax;
g_p_grid(i,:) = g_p;
policy_pm_grid(i,:) = policy_pm;

end



figure
plot(tau_grid,tau1_grid(:,1),'k-.','LineWidth',4);
set(gca,'FontSize',16)
xlabel('\tau')

w_gain = zeros(5,num_y);
w_gain_t = zeros(n_grid,num_y);

options = optimset('Display','off');

for i=1:num_y-10
    sqr1 = @(x) interp1(y(:),welfare_p(1,:),x)-welfare_p(n_grid,i);
    sqr2 = @(x) interp1(y(:),welfare_r(1,:,1),x)-welfare_r(n_grid,i,1);
    sqr3 = @(x) interp1(y(:),welfare_r(1,:,2),x)-welfare_r(n_grid,i,2);
    sqr4 = @(x) interp1(y(:),welfare_m(1,:),x)-welfare_m(n_grid,i);
    sqr5 = @(x) interp1(y(:),welfare_all(1,:),x)-welfare_all(n_grid,i);

    w_gain(1,i) = fsolve(sqr1,y(i),options)-y(i);
    w_gain(2,i) = fsolve(sqr2,y(i),options)-y(i);
    w_gain(3,i) = fsolve(sqr3,y(i),options)-y(i);
    w_gain(4,i) = fsolve(sqr4,y(i),options)-y(i);
    w_gain(5,i) = fsolve(sqr5,y(i),options)-y(i);
end
    
for i=num_y-10+1:num_y
    sqr1 = @(x) interp1(y(:),welfare_p(n_grid,:),x)-welfare_p(1,i);
    sqr2 = @(x) interp1(y(:),welfare_r(n_grid,:,1),x)-welfare_r(1,i,1);
    sqr3 = @(x) interp1(y(:),welfare_r(n_grid,:,2),x)-welfare_r(1,i,2);
    sqr4 = @(x) interp1(y(:),welfare_m(n_grid,:),x)-welfare_m(1,i);
    sqr5 = @(x) interp1(y(:),welfare_all(n_grid,:),x)-welfare_all(1,i);

    w_gain(1,i) = y(i)- fsolve(sqr1,y(i),options);
    w_gain(2,i) = y(i)- fsolve(sqr2,y(i),options);
    w_gain(3,i) = y(i)- fsolve(sqr3,y(i),options);
    w_gain(4,i) = y(i)- fsolve(sqr4,y(i),options);
    w_gain(5,i) = y(i)- fsolve(sqr5,y(i),options);
end

for k=2:n_grid
        for i=1:num_y-10
            sqr6 = @(x) interp1(y(:),welfare_all(1,:),x)-welfare_all(k,i);
            w_gain_t(k,i) = fsolve(sqr6,y(i),options) - y(i);
        end
end

for k=2:n_grid
        for i=num_y-10+1:num_y
            sqr6 = @(x) interp1(y(:),welfare_all(k,:),x)-welfare_all(1,i);
            w_gain_t(k,i) = y(i) - fsolve(sqr6,y(i),options);
        end
end


w_gain(:,:) = w_gain(:,:) /(1-beta*rho);


w_gain_t(:,:) = w_gain_t(:,:)/(1-beta*rho);


w_gain_t_sum = zeros(n_grid,1);

for k=2:n_grid
        w_gain_t_sum(k) = sum(w_gain_t(k,:).*mean(g_mat(:,:),2)');

end

figure
plot(tau_grid,w_gain_t_sum,'k-.','LineWidth',4)
xlabel('\tau')
ylabel('compensating variation')
set(gca,'FontSize',16)

