function [f,g_p,g_r,g_m,g_r_w,g_m_w,vv_p,vv_r,vv_m,solution,y,g_mat,tax,policy_pm,policy_prm] = ...
    revenue(tau1,tau,rev_data,disp_op)
beta    = 0.95;
rho     = 0.6661;
sm      = 1d-7;      % small number for log or devide
sm1     = 1d-7;      % convergence criterion for value function iteration
sm2     = 1d-7;      % convergence criterion for SS distribution iteration
sm3     = 1d-8;      % convergence criterion for fsolve
num_y_s = 50;        % number of income grids
num_y   = 100;       % number of income grids
num_r   = 2;
max_w   = 50;        % maximum value of waiting list
x_guess =  [   19.81
              0.11
              0.26];        
y_low   = 52367.227;
y_tax   = y_low;
p_m       = 1;         % market rent
p_r(1)  = 1-0.5125977; % price of regulated housing type 1
p_r(2)  = 1-0.5125977; % price of regulated housing type 2
k_p     = 0.0990;
k_r     = [0.2863 0.2857]; % fraction of regulated 1 and 2
k_m     = 1-k_p-sum(k_r);  % fraction of market
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Setting up
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read input parameters

load('test_1hh.mat')

h     = xx(1);
h_r   = xx(2:3);
mu    = xx(4);
sigma = xx(5);
alpha = xx(6);
gamma = xx(7);
h_min = xx(8);

y_min = h_min*p_m;

% algorith stabilization tricks
sigma = max(sm,sigma);
mu = max(sm,mu);

% Tauchen small grids
m =4;             % coverage m standard deviation
[log_y_s,g_tran_s] = tauchen(num_y_s,mu,rho,sigma,m); % (log income, transition matrix)
y_s = exp(log_y_s)+y_min;   % income level
tax_s = tau*y_s + tau1*max(0,y_s - y_tax);
% define flow utility function
u_p_s = (y_s - tax_s).^(1-alpha).*(h-h_min).^alpha;            % public
u_r_s = zeros(num_y_s,num_r);                                            % regulated
u_m_s = alpha^alpha*(1-alpha)^(1-alpha)*(y_s-h_min*p_m)/p_m^alpha; % mover
h_m_s = alpha*(y_s-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s_s = zeros(num_y_s,num_y_s);                                      % stayer

for i=1:num_y_s
    for j=1:num_y_s
        if y_s(i)-p_m*h_m_s(j) < sm
            u_s_s(i,j) = -1000000000;
        else
            u_s_s(i,j) = (y_s(i)-p_m*h_m_s(j))^(1-alpha)*(h_m_s(j)-h_min)^alpha;
        end
    end
end

for i_r=1:num_r
    for i_y=1:num_y_s
        if y_s(i_y)-p_r(i_r)*h_r(i_r) < sm
            u_r_s(i_y,i_r) = -1000000000;
        else
            u_r_s(i_y,i_r) = (y_s(i_y)-p_r(i_r)*h_r(i_r))^(1-alpha)*(h_r(i_r)-h_min)^alpha;
        end
    end
end
 
%Tauchen approximation of AR1
m =4;             % coverage m standard deviation
[log_y,g_tran] = tauchen(num_y,mu,rho,sigma,m); % (log income, transition matrix)
y = exp(log_y)+y_min;   % income level

tax = tau*y + tau1*max(0,y - y_tax);

y_cdf = zeros(num_y,num_y); % CDF of income process
for i=1:num_y
    y_cdf(:,i) = sum(g_tran(:,1:i),2);
end

g_mat = g_tran';

% define flow utility function
u_p = (y-tax).^(1-alpha).*(h-h_min).^alpha;            % public
u_r = zeros(num_y,num_r);                                            % regulated
u_m = alpha^alpha*(1-alpha)^(1-alpha)*(y-h_min*p_m)/p_m^alpha; % mover
h_m = alpha*(y-h_min*p_m)/p_m+h_min;                           % optimal housing consumption for movers with each income
u_s = zeros(num_y,num_y);                                      % stayer

for i=1:num_y
    for j=1:num_y
        if y(i)-p_m*h_m(j)< sm
            u_s(i,j) = -1000000000;
        else
            u_s(i,j) = (y(i)-p_m*h_m(j))^(1-alpha)*(h_m(j)-h_min)^alpha;
        end
    end
end

for i_r = 1:num_r
    for i_y=1:num_y
        if y(i_y)-p_r(i_r)*h_r(i_r)< sm
            u_r(i_y,i_r) = -1000000000;
        else
            u_r(i_y,i_r) = (y(i_y)-p_r(i_r)*h_r(i_r))^(1-alpha)*(h_r(i_r)-h_min)^alpha;
        end
    end
end
 

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Find steady state pw, w_bar, qr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SS = @(x) flow_difference_1hh(x, sm, sm1, sm2, gamma,u_p,u_r,u_m,u_s,u_p_s,u_r_s,u_m_s,u_s_s,g_tran,g_tran_s,g_mat,y,y_s,h_m,h_m_s,k_p,k_r,k_m,num_y_s,num_y,num_r,max_w,y_low);

options = optimset('Display',disp_op,'TolFun',sm3,'TolX',sm3,'MaxFunEvals',200);

[xx,ff,exitflag] = fsolve(SS,x_guess,options);

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(2:3) = xx(2:3) +  0.01;
       
      [xx,~,~] = fsolve(SS,xx,options);
end

[~,g_p,g_r,g_m,g_r_w,g_m_w,policy_pm,policy_prm,~,~,~,~,~,vv_p,vv_r,vv_m]= SS(xx);

solution = xx;


f = sum(tax.*g_p)-rev_data;




