%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0. Basic settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; 
clc;
disp_op    = 'off';  % show fsolve iteration
beta    = 0.95;
sm      = 1d-7;      % small number for log or devide
sm1     = 1d-7;      % convergence criterion for value function iteration
sm2     = 1d-7;      % convergence criterion for SS distribution iteration
sm3     = 1d-8;      % convergence criterion for fsolve
num_y_s = 50;        % number of income grids
num_y   = 100;       % number of income grids
max_w   = 50;        % maximum value of waiting list
n_type = 2;
num_r   = 2;
n_burn  = 50;
x_guess =  [ 19.1146176175051         0.102773314130108         0.225056244618774];        
y_low = [52449.946 52264.468];
p_r(1)  = 1-0.5125977; % price of regulated housing type 1
p_r(2)  = 1-0.5125977; % price of regulated housing type 2
k_p_0(1) = 0.072629052;
k_r_0(1,:) = [0.138655462	0.165666267];
k_m_0(1) = 0.177070828;
k_p_0(2) = 0.026410564;
k_r_0(2,:) = [0.147659064	0.120048019] ;
k_m_0(2) = 0.151860744;
k_pmr = k_p_0+sum(k_r_0,2)'+k_m_0;
k_rr = sum(k_r_0,1);
k_mm = sum(k_m_0);
p_m=1;
% generate randome sample 
n_sim         = 2500;      % number of simulation sample
n_t           = 100;       % length of simulation
rng(10) % random number seed
qt_draw = rand(n_sim,n_t);
pt_draw = rand(n_sim,n_t);
yt_draw = rand(n_sim,n_t);
% read data and parameter values
data = xlsread('moment_2hh.xlsx')';
load('test_2hh.mat')
x=xx;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Setting up
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read input parameters
rho   = zeros(n_type,1);
mu    = zeros(n_type,1);
sigma = zeros(n_type,1);
h_r   = zeros(n_type,num_r);
h     = zeros(n_type,1);
alpha = zeros(n_type,1);
gamma = zeros(n_type,1);
h_min = zeros(n_type,1);

for j=1:n_type
    h(j)     = x(1);
    h_r(j,:) = x(2:3);
    mu(j)    = x(4+(j-1)*5);
    sigma(j) = x(5+(j-1)*5);
    alpha(j) = x(6+(j-1)*5);
    gamma(j) = x(7+(j-1)*5);
    h_min(j) = x(8+(j-1)*5);
end

rho(1) = 0.6306; % female
rho(2) = 0.7013; % male

y_min = h_min*p_m;

% algorith stabilization tricks
sigma = max(sm,sigma);
mu = max(sm,mu);

% Tauchen : small number of grids for value function iteration
m =4;             % coverage m standard deviation
y_s     = zeros(num_y_s,n_type);
log_y_s = zeros(num_y_s,n_type);
g_all_s = zeros(num_y_s,num_y_s,n_type);
for j= 1:n_type
    [log_y_s(:,j),g_all_s(:,:,j)] = tauchen(num_y_s,mu(j),rho(j),sigma(j),m); % (log income, transition matrix)
y_s(:,j) = exp(log_y_s(:,j))+ y_min(j);
end


% Tauchen : large number of grids for other steps
m =4;             % coverage m standard deviation
y   = zeros(num_y,n_type);
log_y = zeros(num_y,n_type);
g_all = zeros(num_y,num_y,n_type);
for j= 1:n_type
    [log_y(:,j),g_all(:,:,j)] = tauchen(num_y,mu(j),rho(j),sigma(j),m); % (log income, transition matrix)
y(:,j) = exp(log_y(:,j)) + y_min(j);   % income level
end


y_cdf = zeros(num_y,num_y,n_type); % CDF of income process
for j=1:n_type
    for i=1:num_y
        y_cdf(:,i,j) = sum(g_all(:,1:i,j),2);
    end
end
g_mat = permute(g_all,[2 1 3]);

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Find steady state pw, w_bar, qr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SS = @(x) flow_difference_2hh(x, sm2, sm, sm1, num_y,num_r,n_type, max_w, k_p_0, k_r_0, k_m_0,...
    g_mat, g_all, y, y_low, alpha, h, h_r, p_r,k_pmr,k_rr,k_mm,num_y_s, g_all_s, y_s,gamma,h_min,p_m);
options = optimset('Display',disp_op,'TolFun',sm3,'TolX',sm3,'MaxFunEvals',100);
disp('---------------------------------------------------------------------')
disp('solve model')
disp('---------------------------------------------------------------------')
[xx,ff,exitflag] = fsolve(SS,x_guess,options);

if (norm(ff)^2>1d-10) || (exitflag < 1)
    
      xx(2:3) = xx(2:3) +  0.01;
       
      [xx,ff,exitflag] = fsolve(SS,xx,options);
end

[~,g_p,g_r,g_m,g_r_w,g_m_w,policy_pm,policy_prm,policy_pms,policy_prms,policy_rms,policy_ms,policy_rm,...
     vv_p,vv_r,vv_m,k_p,k_r,k_m] =SS(xx);
w_bar = zeros(1,n_type);
pw    = zeros(1,n_type);
xx(1) = min(max_w,max(1,xx(1)));
for j=1:n_type
w_bar(j) = floor(xx(1));
pw(j) = 1-(xx(1)-w_bar(j));
end
qr = xx(2:num_r+1);
qr = min(1,max(0,qr));

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Simulate moment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%w
disp('---------------------------------------------------------------------')
disp('simulate')
disp('---------------------------------------------------------------------')

yt = zeros(n_sim,n_t); % income
dt = zeros(n_sim,n_t); % previous choice: 1:public, 2:regulated, 3:unregulated
ht = zeros(n_sim,n_t); % previous housing size, relavant if dt = 3
wt = zeros(n_sim,n_t); % waiting time: 0,...,w_bar+1, not relavant if dt = 1
lt = zeros(n_sim,n_t); % length of stay
ayt = zeros(n_sim,n_t);
qt = 1*(qt_draw <= qr(1)) + 2*(qt_draw > qr(1)).*(qt_draw <= sum(qr));

model = zeros(20,n_type);

for x = 1:n_type

k_all = k_p(x) + sum(k_r(x,:)) + k_m(x);
   
pt = pt_draw <= pw(x);
% initialize

ct = 0;
% regulated housing
for i_r = 1:num_r
    for j=0:w_bar+1

        num_g = round(n_sim*k_r(x,i_r)*g_r_w(j+1,x,i_r));

        for i= ct+1:ct+num_g

        yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_r(:,j+1,x,i_r)))+1;
        dt(i,1) = 2;
        wt(i,1) = j;
        qt(i,1) = i_r;
        end
        ct = ct + num_g;    
    end
end

% unregulated housing

g_m_h_w = zeros(num_y, w_bar(x)+2);
g_m_y = zeros(num_y,num_y,w_bar(x)+2);

for j=0:w_bar(x)+1
    for i_h=1:num_y
        g_m_h_w(i_h,j+1) = sum(g_m(:,i_h,j+1,x))*g_m_w(j+1,x);
        temp = sum(g_m(:,i_h,j+1,x));
        g_m_y(:,i_h,j+1) = g_m(:,i_h,j+1,x)/temp;
        num_g = round(n_sim*k_m(x)*g_m_h_w(i_h,j+1));
        
        for i= ct+1:min(n_sim,ct+num_g)
            yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_m_y(:,i_h,j+1)))+1;
            dt(i,1) = 3;
            ht(i,1) = i_h;
            wt(i,1) = j;
        end
        ct = ct + num_g;    
    end
end


ct = min(n_sim-1,ct);

% public housing
dt(ct+1:n_sim,1) = 1;

for i = ct+1:n_sim
    yt(i,1) = sum(yt_draw(i,1) >= cumsum(g_p(:,x)))+1;
end

     
yt(:,1) = min(num_y, yt(:,1)); 
yt(:,1) = max(1, yt(:,1));
ht(:,1) = max(1, ht(:,1));
 
% forward simulation
for t = 1:n_t-1
    for i=1:n_sim
        
         % with public housing options and no regulated housing option : policy_pm(yt)          
         if    (dt(i,t) == 1 && qt(i,t)==0 ) 
             
                if policy_pm(yt(i,t),x) == 1
                    dt(i,t+1) = 1;
                else
                    dt(i,t+1) = 3;
                    wt(i,t+1) = 0;
                end                           
         
         % with public and regulated housing options : policy_prm(yt,dt): h1     
         elseif (dt(i,t) == 1 && qt(i,t)>=1 ) ... 
             || (dt(i,t) == 2 && wt(i,t) == w_bar(x)   && pt(i,t)==1  && y(yt(i,t),x) <= y_low(x)) ...
             || (dt(i,t) == 2 && wt(i,t) == w_bar(x)+1 && y(yt(i,t),x) <= y_low(x))
             
                 if policy_prm(yt(i,t),dt(i,t),x,qt(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_prm(yt(i,t),dt(i,t),x,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    wt(i,t+1) = 0;
                    qt(i,t+1) = qt(i,t);
                 else
                    dt(i,t+1) = 3;
                    wt(i,t+1) = 0;
                 end
                 
                 
         % policy_pms(yt,ht): dt= 3 by construction         
         elseif (dt(i,t) == 3 && wt(i,t) == w_bar(x)   && pt(i,t)==1  && qt(i,t)==0 && y(yt(i,t),x) <= y_low(x)) ...
             || (dt(i,t) == 3 && wt(i,t) == w_bar(x)+1 && qt(i,t)==0 && y(yt(i,t),x) <= y_low(x))
             
                 if policy_pms(yt(i,t),ht(i,t),x) == 1
                    dt(i,t+1) =  1;
                 elseif policy_pms(yt(i,t),ht(i,t),x) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                    wt(i,t+1) = 0;
                 end    
                 
                 
         % policy_prms(yt,ht): dt= 3 by construction         
         elseif (dt(i,t) == 3 && wt(i,t) == w_bar(x)   && pt(i,t)==1  && qt(i,t)>=1 && y(yt(i,t),x) <= y_low(x)) ...
             || (dt(i,t) == 3 && wt(i,t) == w_bar(x)+1 && qt(i,t)>=1 && y(yt(i,t),x) <= y_low(x))
             
                 if policy_prms(yt(i,t),ht(i,t),x,qt(i,t)) == 1
                    dt(i,t+1) =  1;
                 elseif policy_prms(yt(i,t),ht(i,t),x,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    wt(i,t+1) = 0;
                    qt(i,t+1) = qt(i,t);
                 elseif policy_prms(yt(i,t),ht(i,t),x,qt(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                    wt(i,t+1) = 0;
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                    wt(i,t+1) = 0;
                 end     
                 
         % policy_rms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)>=1 && y(yt(i,t),x) <= y_low(x)) 
             
                 if policy_rms(yt(i,t),ht(i,t),wt(i,t)+1,x,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1)= qt(i,t);
                 elseif policy_rms(yt(i,t),ht(i,t),wt(i,t)+1,x,qt(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = wt(i,t)+1;
                 
         % policy_ms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==0 && y(yt(i,t),x) <= y_low(x)) 
             
                 if policy_ms(yt(i,t),ht(i,t),wt(i,t)+1,x) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = wt(i,t)+1;      
                 
         % policy_rm(yt,dt,wt)   
         elseif  (dt(i,t) == 2  && y(yt(i,t),x) <= y_low(x))
                 if policy_rm(yt(i,t),dt(i,t),wt(i,t)+1,x,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1) = qt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 end                        
                 wt(i,t+1) = wt(i,t)+1;       
                 
                 
         % policy_rms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)>=1 && y(yt(i,t),x) > y_low(x)) 
             
                 if policy_rms(yt(i,t),ht(i,t),1,x,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1) = qt(i,t);
                 elseif policy_rms(yt(i,t),ht(i,t),1,x,qt(i,t)) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = 0;
                 
         % policy_ms(yt,ht,wt): dt= 3 by construction         
         elseif (dt(i,t) == 3  && qt(i,t)==0 && y(yt(i,t),x) > y_low(x)) 
             
                 if policy_ms(yt(i,t),ht(i,t),1,x) == 3
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = ht(i,t);
                 end       
                 
                 wt(i,t+1) = 0;     
                 
         % policy_rm(yt,dt,wt)   
         elseif  (dt(i,t) == 2  && y(yt(i,t),x) > y_low(x)) 
             
                 if policy_rm(yt(i,t),dt(i,t),1,x,qt(i,t)) == 2
                    dt(i,t+1) = 2;
                    qt(i,t+1) = qt(i,t);
                 else
                    dt(i,t+1) = 3;
                    ht(i,t+1) = yt(i,t);
                 end                        
                 wt(i,t+1) = 0;                   
               
         end         

        % update duration
         
         if (dt(i,t) <= 2 && dt(i,t+1) == dt(i,t)) || (dt(i,t) == 3 && dt(i,t+1) == dt(i,t) && ht(i,t+1) == ht(i,t))
             lt(i,t+1) = lt(i,t)+1;
         else
             lt(i,t+1) = 0;
         end
         
         % update income
         
         yt(i,t+1) = sum(yt_draw(i,t) >= y_cdf(yt(i,t),:,x))+1;   
               
    end  
    yt(:,t+1) = min(num_y, yt(:,t+1));  
    yt(:,t+1) = max(1, yt(:,t+1));
    ht(:,t+1) = max(1,ht(:,t+1));
    
end
%% calculate moment

for i=1:n_sim
    for j=1:n_t
        ayt(i,j) = y(yt(i,j),x);
    end
end


%toc;tic
 
    dt_s = dt(:,n_burn:n_t);
    qt_s = qt(:,n_burn:n_t);
    lt_s= lt(:,n_burn:n_t);
    ayt_s = ayt(:,n_burn-1:n_t-1);
    %ayt_s = ayt(:,n_burn:n_t);
    ht_s  = (h_min(x) + alpha(x)*(ayt_s-h_min(x)*p_m));
    %ayt_s = ayt(:,n_t-n_burn:n_t); 

    model(1,x) = mean(ayt_s(dt_s==1));
    model(2,x) = mean(ayt_s(dt_s==2 & qt_s==1));
    model(3,x) = mean(ayt_s(dt_s==2 & qt_s==2));
    model(4,x) = mean(ayt_s(dt_s==3));
    model(5,x) = mean(lt_s(dt_s==1));
    model(6,x) = mean(lt_s(dt_s==2 & qt_s==1));
    model(7,x) = mean(lt_s(dt_s==2 & qt_s==2));
    model(8,x) = mean(lt_s(dt_s==3));
    model(9,x) = std(ayt_s(dt_s==1));
    model(10,x) = h_r(x,1)/12*p_r(1);
    model(11,x) = h_r(x,2)/12*p_r(2);
    model(12,x)=mean(ht_s(dt_s==3 & ayt_s>=75000 & ayt_s<100000))/12;
    model(13,x)=mean(ht_s(dt_s==3 & ayt_s>=100000 & ayt_s<125000))/12;
    model(14,x)=mean(ht_s(dt_s==3 & ayt_s>=125000 & ayt_s<150000))/12;
    model(15,x) = sum(sum((ayt_s>=y_low(x)).*(dt_s==1)))/sum(sum((dt_s==1)));
    model(16,x) = mean(ayt_s(dt_s==1 & ayt_s>=y_low(x)))/y_low(x)-1;
    model(17,x) = k_p(x)/k_pmr(x);
    model(18,x) = k_r(x,1)/k_pmr(x);
    model(19,x) = k_r(x,2)/k_pmr(x);
    model(20,x) = k_m(x)/k_pmr(x);
    
end
    
data2 = reshape(data, [1,20*n_type]);
model2 = reshape(model, [1,20*n_type]);  
 

format longG
disp('tabble 3')
disp([alpha h_min gamma mu sigma rho h h_r]')
disp('tabble 4')
disp([w_bar(1) pw(1) qr]')


