clear all
set more off

use nychvs_2011.dta, clear

recode UF17B_1 (1=1) (nonmiss=0), gen(section_8)  
recode TOT_PER (1=45850) (2=52400) (3=58950) (4=65450) (5=70700) (6=75950) (7=81200) (8=86400) , gen(incomelimit)

********************************************************************************
* trim data
********************************************************************************

* missing values 
replace UF42 =0 if UF42==9999999 // no income
replace UF34 =0 if UF34==9999999 // no income
replace UF50 =. if UF50==98      // floor of unit not reported
replace UF29 =. if UF29==0000    // contract housing share not computed
replace UF29 =. if UF29>101      // not applicable
replace UF30 =. if UF30==0000    // gross    housing share not computed
replace UF30 =. if UF30>101      // not applicable   
replace UF17 =. if UF17==99999   // contract rent not applicable
replace UF26 =. if UF26==9999    // gross rent not applicable
replace UF17A=. if UF17A==99998 | UF17A==99999 // out of pockect rent not reported or not applicable
replace SC23 =. if SC23==8       // condition of building not reported
replace SC114=. if SC114==4      // condo coop not known
replace UF17 =. if New_CSR==5
replace UF26 =. if New_CSR==5




* define housing quality
recode SC149 (1=1) (2=0), gen(elevator)
recode SC152 (0=1) (nonmiss = 0), gen(complete_plumbing)
recode SC155 (0=1) (nonmiss = 0), gen(complete_kitchen)
recode SC188 (1=1) (2=0) (8=.), gen(mice)
recode SC185 (0=1) (1=0) (8=.), gen(heat_break)
recode SC24  (1=1) (2=0) (8=.), gen(bad_neighbor)
recode UF1_5  (9= 1) (1=0) (8=.), gen(bad_wall)
recode UF1_10 (9= 1) (1=0) (8=.), gen(bad_window)

rename UF23 bldg_age 
rename UF48 bldg_size  // number of units in bldg
rename UF50 bldg_floor // floor of unit
rename SC23 bldg_cond  // Condition of Building (Dilapidated, Soud, Deterorating, Not reported)
rename SC114 bldg_type // Condo/Coop Status (No condo, coop, Don't know)
rename SC158 bldg_fuel // Type of Heating Fuel (fuel oil, utility gas, electricity, other fuel)
rename UF11 bldg_story // Stories in bldg

* define regional index

gen region = 100*REC14 + CD

* define variables
gen monthrent = UF17
gen logrent = log(monthrent)
gen rentratio = UF29/100


* define limit eligible hh
local limit = 200 // eligibility criterion:  150 percent of median income

* inflation adjustment
gen income = UF42

gen rentratio2 = monthrent*12/income

gen eligible = .
replace eligible = 1 if income <= incomelimit*`limit'/80

* 80% limit
gen eligible4=0
replace eligible4 = 1 if income <= incomelimit
replace eligible4 = 1-eligible4
gen exceed4 = (income-incomelimit)/incomelimit if income > incomelimit


* define housing status
recode New_CSR (1 2 12 86 =1 )(nonmiss=0), gen(owner) // owners

gen     phtype2=.
replace phtype2=1 if New_CSR ==5                      // public housing
replace phtype2=1 if New_CSR ~=5 & section_8 == 1     // voucher
replace phtype2=2 if section_8~=1  & New_CSR==30      // stablized pre 1947
replace phtype2=2 if section_8~=1  & New_CSR==31      //           post
replace phtype2=2 if section_8~=1  & New_CSR==21      // HUD regulated
replace phtype2=2 if section_8~=1  & New_CSR==85      // Mitchell lama rental
replace phtype2=3 if section_8~=1  & New_CSR==80      // market rate
replace phtype2=3 if section_8~=1  & owner == 1   // owners

recode phtype2 (1=1) (nonmiss=0) , gen (public)
recode phtype2 (2=1) (nonmiss=0) , gen (regulated)
recode phtype2 (3=1) (nonmiss=0) , gen (market)

label define phtypel2  1 "&Public&"
label define phtypel2  2 "&Regulated&", add 
label define phtypel2  3 "&Market rate&", add
label values phtype2 phtypel2 

* define sample for analysis

* drop old
drop if  HHR3T>=62

* drop rich hh
drop if eligible==.

* boro selection
keep if REC14 == 3 // choose boro

* drop owner

drop if owner==1

drop if section_8==1 

* drop no income HH


drop if income<1

* drop small programs 
drop if New_CSR == 20 | New_CSR == 95 | New_CSR == 90 // drop Article 4 or 5, in Rem, rent controll respectively


gen num_bed = SC151-1
gen num_bed2 = num_bed^2

gen room = SC150
gen room2 = room^2

gen other_room = room - num_bed
gen other_room2 = other_room^2

*la var public "Public"
la var regulated "regulated"
la var num_bed "bed room"
la var other_room "other room"
la var complete_plumbing "complete plumbing"
la var complete_kitchen "complete kitchen"
la var bad_neighbor "bad neighbor"
la var bad_wall "bad wall"
la var bad_window "bad window"
la var mice "mice"
la var elevator "elevator"
la var section_8 "section 8"

* HH characteristics
gen kids6 = REC8-1
gen kids6_2 = kids6^2
gen kids18 = REC7-1 
gen kids18_2 = kids18^2
gen adult = TOT_PER - kids18
gen adult2 = adult^2

gen numper = TOT_PER
gen numper2 = numper^2

gen income2  = income^2
gen lincome = log(max(1,income))

gen numyear = 2011 - SC52
gen numyear2 = numyear^2

replace numyear=. if owner==1

gen age = HHR3T
gen age2 = age^2

recode HHR2 (1=0) (2=1), gen(female)

* define working status
gen wageratio = UF34/max(0.0001, UF42)
recode wageratio (0.5/1=1) (nonmiss =0), gen(working)
* define regression setting

***************************************
** Table 1
***************************************
tab phtype2 
tabstat monthrent numyear income female kids18 working, by(phtype2)


local myvars_condition num_bed other_room complete_kitchen complete_plumbing   elevator   bad_wall   // income age  numyear // income2 age2 numyear2 // bad_neighbor mice  //kids18  numper

gen hhtype = female
replace hhtype = 2 if hhtype==0

***************************************
** Table 2
***************************************

regress logrent regulated `myvars_condition' i.bldg_age i.bldg_story i.bldg_floor i.bldg_fuel  i.bldg_cond  i.bldg_size i.bldg_type if phtype2>=2 & phtype2<=3 
*outreg2 using hedonic,   replace side paren label dec(3) tex ctitle("No Sub-boro Control")
regress logrent regulated `myvars_condition' i.region  i.bldg_age i.bldg_story i.bldg_floor i.bldg_fuel  i.bldg_cond  i.bldg_size i.bldg_type if phtype2>=2 & phtype2<=3 
*outreg2 using hedonic,   append side paren label dec(3) tex ctitle("With Sub-boro Control")


* predict housing quality
predict lhat1
replace lhat1=logrent if lhat1==. // impute using rents if predicted value is not avaialble
gen hat1 = exp(lhat1)/(1-0.5125977)*12
****************************************
** figure 8
****************************************

hist hat1 if phtype2==2, xtitle(housing quality)
